@@ -1,21 +1,15 @@
 #!/usr/bin/perl -T
-#############################################################################
-# Open WebMail - Provides a web interface to user mailboxes                 #
-#                                                                           #
-# Copyright (C) 2001-2002                                                   #
-# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung   #
-# Copyright (C) 2000                                                        #
-# Ernie Miller  (original GPL project: Neomail)                             #
-#                                                                           #
-# This program is distributed under GNU General Public License              #
-#############################################################################
+#
+# openwebmail-abook.pl - address book program
+#
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
+$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
 umask(0002); # make sure the openwebmail group can write
 
@@ -25,13 +19,13 @@ use CGI qw(-private_tempfiles :standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "openwebmail-shared.pl";
+require "ow-shared.pl";
 require "filelock.pl";
 
 use vars qw(%config %config_raw);
 use vars qw($thissession);
 use vars qw($loginname $domain $user $userrealname $uuid $ugid $homedir);
-use vars qw(%prefs %style);
+use vars qw(%prefs %style %icontext);
 use vars qw($folderdir @validfolders $folderusage);
 use vars qw($folder $printfolder $escapedfolder);
 
@@ -65,6 +59,8 @@ if ($action eq "addressbook") {
    clearaddress();
 } elsif ($action eq "importabook") {
    importabook();
+} elsif ($action eq "importabook_pine") {
+   importabook_pine();
 } elsif ($action eq "exportabook") {
    exportabook();
 } else {
@@ -85,13 +81,7 @@ sub addressbook {
    my $html = '';
    my $temphtml;
 
-   open (ABOOKTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/addressbook.template") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/addressbook.template");
-   while (<ABOOKTEMPLATE>) {
-      $html .= $_;
-   }
-   close (ABOOKTEMPLATE);
-
+   $html=readtemplate("addressbook.template");
    $html = applystyle($html);
 
    if (defined($lang_text{$field})) {
@@ -150,7 +140,7 @@ sub addressbook {
    # split $preexisting in to a hash
    my %preexistinghash=();
    foreach my $u (str2list($preexisting,0)) {
-      my ($name, $email)=email2nameaddr($u);
+      my $email=(email2nameaddr($u))[1];
       $preexistinghash{$email}=$u;
    }
 
@@ -189,31 +179,6 @@ sub addressbook {
          close (ABOOK);
       }
 
-      # import pine addressbook
-      if (open (ABOOK,"$homedir/.addressbook") ) {
-         while (<ABOOK>) {
-            my ($dummy, $name, $email, $dummy2, $note) = split(/\t/, $_, 5);
-            chomp($email);
-            chomp($note);
-            if ( $abook_keyword ne "" &&
-                 ( ($abook_searchtype eq "name" &&
-                    $name!~/$abook_keyword/i && $name!~/\Q$abook_keyword\E/i)   ||
-                   ($abook_searchtype eq "email" &&
-                    $email!~/$abook_keyword/i && $email!~/\Q$abook_keyword\E/i) ||
-                   ($abook_searchtype eq "note" &&
-                    $note!~/$abook_keyword/i && $note!~/\Q$abook_keyword\E/i)   ||
-                   ($abook_searchtype eq "all" &&
-                    "$name.$email.$note" !~ /$abook_keyword/i &&
-                    "$name.$email.$note" !~ /\Q$abook_keyword\E/i) )  ) {
-               next;
-            }
-            next if ($email=~/^\s*$/);	# skip if email is null
-            $addresses{"$name"} = "$email";
-            $notes{"$name"} = "$note";
-         }
-         close (ABOOK);
-      }
-
       foreach my $name (sort { lc($a) cmp lc($b) } keys %addresses) {
          my $email=$addresses{$name};
          my $emailstr;
@@ -385,7 +350,7 @@ sub importabook {
 #            openwebmailerror(qq|$lang_err{'abook_invalid'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;firstmessage=$firstmessage&amp;message_id=$escapedmessageid">$lang_err{'back'}</a> $lang_err{'tryagain'}|);
 #         }
 #      }
-      unless ( -f "$folderdir/.address.book" ) {
+      if (! -f "$folderdir/.address.book" ) {
          open (ABOOK, ">>$folderdir/.address.book"); # Create if nonexistent
          close(ABOOK);
       }
@@ -465,13 +430,7 @@ sub importabook {
       my $html = '';
       my $temphtml;
 
-      open (IMPORTTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/importabook.template") or
-         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/importabook.template");
-      while (<IMPORTTEMPLATE>) {
-         $html .= $_;
-      }
-      close (IMPORTTEMPLATE);
-
+      $html=readtemplate("importabook.template");
       $html = applystyle($html);
 
       printheader();
@@ -550,6 +509,66 @@ sub importabook {
       printfooter(2);
    }
 }
+
+sub importabook_pine {
+   if ( ! -f "$folderdir/.address.book" ) {
+      open (ABOOK, ">>$folderdir/.address.book"); # Create if nonexistent
+      close(ABOOK);
+   }
+
+   if (open (PINEBOOK,"$homedir/.addressbook") ) {
+      my ($name, $email, $note);
+      my (%addresses, %notes);
+      my $abooktowrite='';
+
+      filelock("$folderdir/.address.book", LOCK_EX|LOCK_NB) or
+         openwebmailerror("$lang_err{'couldnt_lock'} $folderdir/.address.book!");
+      open (ABOOK,"+<$folderdir/.address.book") or
+         openwebmailerror("$lang_err{'couldnt_open'} $folderdir/.address.book!");
+
+      while (<ABOOK>) {
+         ($name, $email, $note) = split(/\@\@\@/, $_, 3);
+         chomp($email); chomp($note);
+         $addresses{"$name"} = $email;
+         $notes{"$name"}=$note;
+      }
+
+      while (<PINEBOOK>) {
+         my ($name, $email, $note) = (split(/\t/, $_,5))[1,2,4];
+         chomp($email);
+         chomp($note);
+         next if ($email=~/^\s*$/);  # skip if email is null
+         $addresses{"$name"} = $email;
+         $notes{"$name"}=$note;
+      }
+      close (PINEBOOK);
+
+      seek (ABOOK, 0, 0) or
+         openwebmailerror("$lang_err{'couldnt_seek'} $folderdir/.address.book!");
+
+      foreach (sort keys %addresses) {
+         ($name,$email,$note)=($_, $addresses{$_}, $notes{$_});
+         $name=~s/\@\@/\@\@ /g; $name=~s/\@$/\@ /;
+         $email=~s/\@\@/\@\@ /g; $email=~s/\@$/\@ /;
+         $abooktowrite .= "$name\@\@\@$email\@\@\@$note\n";
+      }
+
+      if (length($abooktowrite) > ($config{'maxbooksize'} * 1024)) {
+         openwebmailerror(qq|$lang_err{'abook_toobig'}|.
+                          qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;firstmessage=$firstmessage&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>|.
+                          qq|$lang_err{'tryagain'}|);
+      }
+      print ABOOK $abooktowrite;
+      truncate(ABOOK, tell(ABOOK));
+
+      close (ABOOK) or openwebmailerror("$lang_err{'couldnt_close'} $folderdir/.address.book!");
+      filelock("$folderdir/.address.book", LOCK_UN);
+
+      writelog("import addressbook");
+      writehistory("import addressbook");
+   }
+   editaddresses();
+}
 #################### END IMPORTABOOK #########################
 
 ##################### EXPORTABOOK ############################
@@ -600,13 +619,7 @@ sub editaddresses {
    my $html = '';
    my $temphtml;
 
-   open (EDITABOOKTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/editaddresses.template") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/editaddresses.template");
-   while (<EDITABOOKTEMPLATE>) {
-      $html .= $_;
-   }
-   close (EDITABOOKTEMPLATE);
-
+   $html=readtemplate("editaddresses.template");
    $html = applystyle($html);
 
    if ( -f "$folderdir/.address.book" ) {
@@ -641,15 +654,18 @@ sub editaddresses {
    $html =~ s/\@\@\@FREESPACE\@\@\@/$freespace/g;
 
    if ( param("message_id") ) {
-      $temphtml = qq|<a href="$config{'ow_cgiurl'}/openwebmail-read.pl?action=readmessage&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$folder&amp;message_id=$escapedmessageid" title="$lang_text{'backto'} $printfolder"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/backtofolder.gif" border="0" ALT="$lang_text{'backto'} $printfolder"></a> &nbsp; |;
+      $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $printfolder", qq|href="$config{'ow_cgiurl'}/openwebmail-read.pl?action=readmessage&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder&amp;message_id=$escapedmessageid"|). qq| &nbsp; \n|;
    } else {
-      $temphtml = qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$folder" title="$lang_text{'backto'} $printfolder"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/backtofolder.gif" border="0" ALT="$lang_text{'backto'} $printfolder"></a> &nbsp; |;
+      $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $printfolder", qq|href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder"|). qq| &nbsp; \n|;
    }
-   $temphtml .= qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$folder&amp;message_id=$escapedmessageid" title="$lang_text{'importadd'}"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/import.gif" border="0" ALT="$lang_text{'importadd'}"></a> |.
-                qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=exportabook&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$folder&amp;message_id=$escapedmessageid" title="$lang_text{'exportadd'}"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/export.gif" border="0" ALT="$lang_text{'exportadd'}"></a> |.
-                qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=clearaddress&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$folder&amp;message_id=$escapedmessageid" onclick="return confirm('$lang_text{'clearadd'}?')" title="$lang_text{'clearadd'}"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/clearaddress.gif" border="0" ALT="$lang_text{'clearadd'}"></a> &nbsp; |;
+   $temphtml .= iconlink("import.gif", $lang_text{'importadd'}, qq|href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder&amp;message_id=$escapedmessageid"|) . qq| \n|;
+   if ( -f "$homedir/.addressbook" ) {
+      $temphtml .= iconlink("import.gif", "$lang_text{'importadd'} (Pine)", qq|href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=importabook_pine&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder&amp;message_id=$escapedmessageid"|). qq| \n|;
+   }
+   $temphtml .= iconlink("export.gif", $lang_text{'exportadd'}, qq|href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=exportabook&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder&amp;message_id=$escapedmessageid"|). qq| \n|.
+                iconlink("clearaddress.gif", $lang_text{'clearadd'}, qq|href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=clearaddress&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder&amp;message_id=$escapedmessageid" onclick="return confirm('$lang_text{'clearadd'}?')"|). qq| &nbsp; \n|;
    if ($abook_keyword ne ''){
-      $temphtml .= qq|<a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$folder&amp;message_id=$escapedmessageid&amp;abook_keyword=" title="$lang_text{'refresh'}"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/refresh.gif" border="0" ALT="$lang_text{'refresh'}"></a>|;
+      $temphtml .= iconlink("refresh.gif", $lang_text{'refresh'}, qq|href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder&amp;message_id=$escapedmessageid&amp;abook_keyword="|). qq|\n|;
    }
 
    $html =~ s/\@\@\@MENUBARLINKS\@\@\@/$temphtml/g;
@@ -726,7 +742,8 @@ sub editaddresses {
                          -default=>'',
                          -size=>'30',
                          -override=>'1');
-   $temphtml .= qq|<a href="Javascript:GoAddressWindow('email')" title="$lang_text{'group'}"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/group.gif" border="0" ALT="$lang_text{'group'}"></a>|;
+
+   $temphtml .= iconlink("group.gif", $lang_text{'group'}, qq|href="Javascript:GoAddressWindow('email')"|);
 
    $html =~ s/\@\@\@EMAILFIELD\@\@\@/$temphtml/;
 
