@@ -1,21 +1,15 @@
 #!/usr/bin/perl -T
-#############################################################################
-# Open WebMail - Provides a web interface to user mailboxes                 #
-#                                                                           #
-# Copyright (C) 2001-2002                                                   #
-# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung   #
-# Copyright (C) 2000                                                        #
-# Ernie Miller  (original GPL project: Neomail)                             #
-#                                                                           #
-# This program is distributed under GNU General Public License              #
-#############################################################################
+#
+# openwebmail-folder.pl - mail folder management program
+#
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
+$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
 umask(0002); # make sure the openwebmail group can write
 
@@ -25,7 +19,7 @@ use CGI qw(-private_tempfiles :standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "openwebmail-shared.pl";
+require "ow-shared.pl";
 require "filelock.pl";
 require "mime.pl";
 require "maildb.pl";
@@ -33,7 +27,7 @@ require "maildb.pl";
 use vars qw(%config %config_raw);
 use vars qw($thissession);
 use vars qw($loginname $domain $user $userrealname $uuid $ugid $homedir);
-use vars qw(%prefs %style);
+use vars qw(%prefs %style %icontext);
 use vars qw($folderdir @validfolders $folderusage);
 use vars qw($folder $printfolder $escapedfolder);
 
@@ -103,21 +97,14 @@ sub editfolders {
    my $html = '';
    my $temphtml;
 
-   open (EDITFOLDERSTEMPLATE, "$config{'ow_etcdir'}/templates/$prefs{'language'}/editfolders.template") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/editfolders.template!");
-   while (<EDITFOLDERSTEMPLATE>) {
-      $html .= $_;
-   }
-   close (EDITFOLDERSTEMPLATE);
-
+   $html=readtemplate("editfolders.template");
    $html = applystyle($html);
 
    $html =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/g;
 
    printheader();
 
-   $temphtml = qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder" title="$lang_text{'backto'} $printfolder"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/backtofolder.gif" border="0" ALT="$lang_text{'backto'} $printfolder"></a>|;
-
+   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $printfolder", qq|href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedfolder"|). qq|\n|;
    $html =~ s/\@\@\@MENUBARLINKS\@\@\@/$temphtml/g;
 
    $temphtml = start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl") .
@@ -217,8 +204,8 @@ sub _folderline {
    my (%HDB, $newmessages, $allmessages, $foldersize);
    my ($folderfile,$headerdb)=get_folderfile_headerdb($user, $currfolder);
 
-   if ( -f "$headerdb$config{'dbm_ext'}" ) {
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
+   if ( -f "$headerdb$config{'dbm_ext'}" && !-z "$headerdb$config{'dbm_ext'}" ) {
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
       dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
       if ( defined($HDB{'ALLMESSAGES'}) ) {
          $allmessages=$HDB{'ALLMESSAGES'};
@@ -233,7 +220,7 @@ sub _folderline {
          $newmessages='&nbsp;';
       }
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
    } else {
       $allmessages='&nbsp;';
       $newmessages='&nbsp;';
@@ -254,8 +241,8 @@ sub _folderline {
                 qq|<td align="center" bgcolor=$bgcolor>|.
                 qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=displayheaders&amp;sessionid=$thissession&amp;sort=$sort&amp;firstmessage=$firstmessage&amp;folder=$escapedcurrfolder">|.
                 qq|$folderstr</a>&nbsp;\n|.
-                qq|<a href="$url" title="$lang_text{'download'} $folderstr"><IMG SRC="$config{'ow_htmlurl'}/images/iconsets/$prefs{'iconset'}/download.gif" align="absmiddle" border="0" ALT="$lang_text{'download'} $folderstr">|.
-                qq|</a></td>\n|.
+                iconlink("download.gif", "$lang_text{'download'} $folderstr", qq|href="$url"|).
+                qq|</td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
                 qq|<td align="center" bgcolor=$bgcolor>&nbsp;$allmessages</td>|.
                 qq|<td align="center" bgcolor=$bgcolor>&nbsp;$foldersize</td>\n|;
@@ -287,7 +274,7 @@ sub _folderline {
                        -override=>'1');
    $temphtml .= "\n";
 
-   my $jsfolderstr=$lang_folders{$currfolder}||$currfolder; 
+   my $jsfolderstr=$lang_folders{$currfolder}||$currfolder;
    $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
    $temphtml .= submit(-name=>"$lang_text{'markread'}",
                        -class=>"medtext",
@@ -317,17 +304,21 @@ sub _folderline {
 ################### MARKREADFOLDER ##############################
 sub markreadfolder {
    my $foldertomark = param('foldername') || '';
-   $foldertomark =~ s/\.\.+//g;
-   $foldertomark =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertomark =~ s!\.\.+/!!g;
+   $foldertomark =~ s!^\s*/!!g;
+   $foldertomark =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertomark =~ /^(.+)$/) && ($foldertomark = $1);
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertomark);
 
    filelock($folderfile, LOCK_EX|LOCK_NB) or openwebmailerror("$lang_err{'couldnt_lock'} $folderfile!");
 
-   update_headerdb($headerdb, $folderfile);
+   if (update_headerdb($headerdb, $folderfile)<0) {
+      filelock($folderfile, LOCK_UN);
+      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
+   }
 
    my (%HDB, %offset, %status);
-   filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
+   filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
    dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
    foreach my $messageid (keys %HDB) {
       next if ( $messageid eq 'METAINFO'
@@ -342,7 +333,7 @@ sub markreadfolder {
       }
    }
    dbmclose(%HDB);
-   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+   filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
 
    my @markids;
    my $tmpfile="/tmp/markread_tmp_$$";
@@ -352,7 +343,10 @@ sub markreadfolder {
 
    filelock("$tmpfile", LOCK_EX);
 
-   update_headerdb($tmpdb, $tmpfile);
+   if (update_headerdb($tmpdb, $tmpfile)<0) {
+      filelock($tmpfile, LOCK_UN);
+      openwebmailerror("$lang_err{'couldnt_updatedb'} $tmpdb$config{'dbm_ext'}");
+   }
 
    foreach my $messageid (sort { $offset{$a}<=>$offset{$b} } keys %offset) {
       my @copyid;
@@ -361,12 +355,20 @@ sub markreadfolder {
    					$folderfile, $headerdb, $tmpfile, $tmpdb) >0 ) {
          update_message_status($messageid, $status{$messageid}."R", $tmpdb, $tmpfile);
          push(@markids, $messageid);
+
+         if ($#markids>=99) { # flush per 100 msgs
+            operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
+            operate_message_with_ids("move", \@markids,
+   					$tmpfile, $tmpdb, $folderfile, $headerdb);
+            @markids=();
+         }
+
       }
    }
    operate_message_with_ids("delete", \@markids, $folderfile, $headerdb);
-   operate_message_with_ids("copy", \@markids,
+   operate_message_with_ids("move", \@markids,
    					$tmpfile, $tmpdb, $folderfile, $headerdb);
-      
+
    filelock("$tmpfile", LOCK_UN);
    filelock($folderfile, LOCK_UN);
    unlink("$tmpdb$config{'dbm_ext'}", $tmpfile);
@@ -383,8 +385,9 @@ sub markreadfolder {
 sub reindexfolder {
    my $recreate=$_[0];
    my $foldertoindex = param('foldername') || '';
-   $foldertoindex =~ s/\.\.+//g;
-   $foldertoindex =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertoindex =~ s!\.\.+/!!g;
+   $foldertoindex =~ s!^\s*/!!g;
+   $foldertoindex =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertoindex =~ /^(.+)$/) && ($foldertoindex = $1);
    my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldertoindex);
 
@@ -396,13 +399,16 @@ sub reindexfolder {
 
    if ( -f "$headerdb$config{'dbm_ext'}" ) {
       my %HDB;
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH);
-      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_SH) if (!$config{'dbmopen_haslock'});
+      dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
       $HDB{'METAINFO'}={'RENEW'};
       dbmclose(%HDB);
-      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+      filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+   }
+   if (update_headerdb($headerdb, $folderfile)<0) {
+      filelock($folderfile, LOCK_UN);
+      openwebmailerror("$lang_err{'couldnt_updatedb'} $headerdb$config{'dbm_ext'}");
    }
-   update_headerdb($headerdb, $folderfile);
 
    if ($recreate) {
       writelog("reindex folder - $foldertoindex");
@@ -421,8 +427,9 @@ sub reindexfolder {
 ################### ADDFOLDER ##############################
 sub addfolder {
    my $foldertoadd = param('foldername') || '';
-   $foldertoadd =~ s/\.\.+//g;
-   $foldertoadd =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertoadd =~ s!\.\.+/!!g;
+   $foldertoadd =~ s!^\s*/!!g;
+   $foldertoadd =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertoadd =~ /^(.+)$/) && ($foldertoadd = $1);
 
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
@@ -476,8 +483,9 @@ sub is_defaultfolder {
 ################### DELETEFOLDER ##############################
 sub deletefolder {
    my $foldertodel = param('foldername') || '';
-   $foldertodel =~ s/\.\.+//g;
-   $foldertodel =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $foldertodel =~ s!\.\.+/!!g;
+   $foldertodel =~ s!^\s*/!!g;
+   $foldertodel =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($foldertodel =~ /^(.+)$/) && ($foldertodel = $1);
 
    # if is INBOX, return to editfolder immediately
@@ -487,14 +495,16 @@ sub deletefolder {
    }
 
    if ( -f "$folderdir/$foldertodel" ) {
+      my $headerdb="$folderdir/$foldertodel";
+      ($headerdb =~ /^(.+)\/(.*)$/) && ($headerdb = "$1/.$2");
       unlink ("$folderdir/$foldertodel",
-              "$folderdir/.$foldertodel$config{'dbm_ext'}",
-              "$folderdir/.$foldertodel.db",
-	      "$folderdir/.$foldertodel.dir",
-              "$folderdir/.$foldertodel.pag",
-              "$folderdir/.$foldertodel.cache",
               "$folderdir/$foldertodel.lock",
-              "$folderdir/$foldertodel.lock.lock");
+              "$folderdir/$foldertodel.lock.lock",
+              "$headerdb$config{'dbm_ext'}",
+              "$headerdb.db",
+	      "$headerdb.dir",
+              "$headerdb.pag",
+              "$headerdb.cache");
 
       writelog("delete folder - $foldertodel");
       writehistory("delete folder - $foldertodel");
@@ -508,8 +518,9 @@ sub deletefolder {
 ################### RENAMEFOLDER ##############################
 sub renamefolder {
    my $oldname = param('foldername') || '';
-   $oldname =~ s/\.\.+//g;
-   $oldname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $oldname =~ s!\.\.+/!!g;
+   $oldname =~ s!^\s*/!!g;
+   $oldname =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($oldname =~ /^(.+)$/) && ($oldname = $1);
 
    if ($oldname eq 'INBOX') {
@@ -518,8 +529,9 @@ sub renamefolder {
    }
 
    my $newname = param('foldernewname');
-   $newname =~ s/\.\.+//g;
-   $newname =~ s/[\s\/\`\|\<\>;]//g; # remove dangerous char
+   $newname =~ s!\.\.+/!!g;
+   $newname =~ s!^\s*/!!g;
+   $newname =~ s/[\s\`\|\<\>;]//g; # remove dangerous char
    ($newname =~ /^(.+)$/) && ($newname = $1);
 
    if (length($newname) > $config{'foldername_maxlen'}) {
@@ -534,13 +546,17 @@ sub renamefolder {
    }
 
    if ( -f "$folderdir/$oldname" ) {
-      rename("$folderdir/$oldname",          "$folderdir/$newname");
-      rename("$folderdir/.$oldname$config{'dbm_ext'}", "$folderdir/.$newname$config{'dbm_ext'}");
-      rename("$folderdir/.$oldname.db",      "$folderdir/.$newname.db");
-      rename("$folderdir/.$oldname.dir",     "$folderdir/.$newname.dir");
-      rename("$folderdir/.$oldname.pag",     "$folderdir/.$newname.pag");
-      rename("$folderdir/.$oldname.cache",   "$folderdir/.$newname.cache");
-      unlink("$folderdir/$oldname.lock", "$folderdir/$oldname.lock.lock");
+      my $oldheaderdb="$folderdir/$oldname";
+      my $newheaderdb="$folderdir/$newname";
+      ($oldheaderdb =~ /^(.+)\/(.*)$/) && ($oldheaderdb = "$1/.$2");
+      ($newheaderdb =~ /^(.+)\/(.*)$/) && ($newheaderdb = "$1/.$2");
+      rename("$folderdir/$oldname",            "$folderdir/$newname");
+      rename("$oldheaderdb$config{'dbm_ext'}", "$newheaderdb$config{'dbm_ext'}");
+      rename("$oldheaderdb.db",                "$newheaderdb.db");
+      rename("$oldheaderdb.dir",               "$newheaderdb.dir");
+      rename("$oldheaderdb.pag",               "$newheaderdb.pag");
+      rename("$oldheaderdb.cache",             "$newheaderdb.cache");
+      unlink("$folderdir/$newname.lock",       "$folderdir/$oldname.lock.lock");
 
       writelog("rename folder - rename $oldname to $newname");
       writehistory("rename folder - rename $oldname to $newname");
@@ -559,12 +575,12 @@ sub downloadfolder {
    my $buff;
 
    if ( -x '/usr/local/bin/zip' ) {
-      $cmd="/usr/local/bin/zip -r - $folderfile |";
+      $cmd="/usr/local/bin/zip -rq - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
    } elsif ( -x '/usr/bin/zip' ) {
-      $cmd="/usr/bin/zip -r - $folderfile |";
+      $cmd="/usr/bin/zip -rq - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
