@@ -1,21 +1,29 @@
 #!/usr/bin/perl -T
-#############################################################################
-# Open WebMail - Provides a web interface to user mailboxes                 #
-#                                                                           #
-# Copyright (C) 2001-2002                                                   #
-# Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung   #
-# Copyright (C) 2000                                                        #
-# Ernie Miller  (original GPL project: Neomail)                             #
-#                                                                           #
-# This program is distributed under GNU General Public License              #
-#############################################################################
+#################################################################
+#                                                               #
+# Open WebMail - Provides a web interface to user mailboxes     #
+#                                                               #
+# Copyright (C) 2001-2002                                       #
+# The Open Webmail Team                                         #
+#                                                               #
+# Copyright (C) 2000                                            #
+# Ernie Miller  (original GPL project: Neomail)                 #
+#                                                               #
+# This program is distributed under GNU General Public License  #
+#                                                               #
+#################################################################
+
+#
+# openwebmail.pl - entry point of openwebmail
+#
 
 use vars qw($SCRIPT_DIR);
-if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-]+\.pl! ) { $SCRIPT_DIR=$1; }
+if ( $ENV{'SCRIPT_FILENAME'} =~ m!^(.*?)/[\w\d\-\.]+\.pl! || $0 =~ m!^(.*?)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1; }
 if (!$SCRIPT_DIR) { print "Content-type: text/html\n\n\$SCRIPT_DIR not set in CGI script!\n"; exit 0; }
 push (@INC, $SCRIPT_DIR, ".");
 
 $ENV{PATH} = ""; # no PATH should be needed
+$ENV{ENV} = "";      # no startup script for sh
 $ENV{BASH_ENV} = ""; # no startup script for bash
 umask(0002); # make sure the openwebmail group can write
 
@@ -26,7 +34,7 @@ use CGI qw(-private_tempfiles :standard);
 use CGI::Carp qw(fatalsToBrowser);
 CGI::nph();   # Treat script as a non-parsed-header script
 
-require "openwebmail-shared.pl";
+require "ow-shared.pl";
 require "filelock.pl";
 
 use vars qw(%config %config_raw);
@@ -47,8 +55,15 @@ if ( $>!=0 && ($config{'use_homedirspools'}||$config{'use_homedirfolders'}) ) {
    print "Content-type: text/html\n\n'$0' must setuid to root"; exit 0;
 }
 
+# check & create mapping table for solar/lunar, b2g, g2b convertion
+if ( ! -f "$config{'ow_etcdir'}/lunar$config{'dbm_ext'}" ||
+     ! -f "$config{'ow_etcdir'}/g2b$config{'dbm_ext'}" ||
+     ! -f "$config{'ow_etcdir'}/b2g$config{'dbm_ext'}" ) {
+   print "Content-type: text/html\n\nPlease execute  '$config{'ow_cgidir'}/openwebmail-tool.pl --init' on server first!"; exit 0;
+}
+
 # validate allowed_serverdomain
-my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
+my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
 if (! is_serverdomain_allowed($httphost) ) {
    print "Content-type: text/html\n\nService is not available for domain  ' $httphost '";
    exit 0;
@@ -66,29 +81,27 @@ if ( ($config{'logfile'} ne 'no') ) {
    chown($>, $mailgid, $config{'logfile'}) if ($fuid!=$>||$fgid!=$mailgid);
 }
 
-%prefs = %{&readprefs};
-%style = %{&readstyle};
-
-($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
-require "etc/lang/$prefs{'language'}";
-
 ####################### MAIN ##########################
 
 if ( param("loginname") && param("password") ) {
    $loginname=param("loginname");
-   if ($loginname!~/\@/ && param("logindomain") ne "") {
-      $loginname .= "\@".param("logindomain");
+   $loginname.='@'.param("logindomain") if ($loginname!~/\@/ && param("logindomain") ne "");
+   if ($config{'case_insensitive_login'}) {
+      $loginname=lc($loginname);
+   } else {
+      $loginname=$1.'@'.lc($2) if ($loginname=~/^(.+)\@(.+)$/);
    }
+
    my $siteconf;
    if ($loginname=~/\@(.+)$/) {
        my $domain=$1;
        if (! is_serverdomain_allowed($domain)) {
           openwebmailerror("Service is not available for domain  ' $domain '");
        }
-       $siteconf="$config{'ow_etcdir'}/sites.conf/$domain";
+       $siteconf="$config{'ow_sitesconfdir'}/$domain";
    } else {
-       my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
-       $siteconf="$config{'ow_etcdir'}/sites.conf/$httphost";
+       my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
+       $siteconf="$config{'ow_sitesconfdir'}/$httphost";
    }
    readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf");
 
@@ -111,18 +124,23 @@ if ( param("loginname") && param("password") ) {
    $virtname=~s!/!.!g; $virtname=~s/^\.+//;
    update_virtusertable("$config{'ow_etcdir'}/$virtname", $config{'virtusertable'});
 
-   # check & create mapping table for chinese BIG5/GB, solar/lunar convertion
-   mkdb_b2g();
-   mkdb_g2b();
-   mkdb_lunar();
+   %prefs = %{&readprefs};
+   %style = %{&readstyle};
+   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
+   require "$config{'ow_langdir'}/$prefs{'language'}";
 
    login();
 
 } else {            # no action has been taken, display login page
-   my $httphost=$ENV{'HTTP_HOST'}; $httphost=~s/:\d+$//;	# remove port number
-   my $siteconf="$config{'ow_etcdir'}/sites.conf/$httphost";
+   my $httphost=lc($ENV{'HTTP_HOST'}); $httphost=~s/:\d+$//;	# remove port number
+   my $siteconf="$config{'ow_sitesconfdir'}/$httphost";
    readconf(\%config, \%config_raw, "$siteconf") if ( -f "$siteconf");
 
+   %prefs = %{&readprefs};
+   %style = %{&readstyle};
+   ($prefs{'language'} =~ /^([\w\d\._]+)$/) && ($prefs{'language'} = $1);
+   require "$config{'ow_langdir'}/$prefs{'language'}";
+
    loginmenu();
 }
 
@@ -131,18 +149,14 @@ exit 0;
 
 ##################### LOGINMENU ######################
 sub loginmenu {
-   printheader(),
    my $html='';
    my $temphtml;
-   open (LOGIN, "$config{'ow_etcdir'}/templates/$prefs{'language'}/login.template") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/login.template!");
-   while (<LOGIN>) {
-      $html .= $_;
-   }
-   close (LOGIN);
 
+   $html=readtemplate("login.template");
    $html = applystyle($html);
 
+   printheader(),
+
    $temphtml = startform(-action=>"$config{'ow_cgiurl'}/openwebmail.pl",
                          -name=>'login');
    if (defined(param('action'))) {
@@ -151,19 +165,35 @@ sub loginmenu {
       $temphtml .= hidden("subject", param('subject')) if (defined(param('subject')));
    }
    $html =~ s/\@\@\@STARTFORM\@\@\@/$temphtml/;
-   $temphtml = textfield(-name=>'loginname',
-                         -default=>'',
-                         -size=>'12',
-                         -onChange=>'focuspwd()',
-                         -override=>'1');
-   $html =~ s/\@\@\@USERIDFIELD\@\@\@/$temphtml/;
+
+   if (cookie("openwebmail-rememberme")) {
+      $temphtml = textfield(-name=>'loginname',
+                            -default=>cookie("openwebmail-loginname"),
+                            -size=>'12',
+                            -onChange=>'focuspwd()',
+                            -override=>'1');
+   } else {
+      $temphtml = textfield(-name=>'loginname',
+                            -default=>'' ,
+                            -size=>'12',
+                            -onChange=>'focuspwd()',
+                            -override=>'1');
+   }
+   $html =~ s/\@\@\@LOGINNAMEFIELD\@\@\@/$temphtml/;
+
    $temphtml = password_field(-name=>'password',
                               -default=>'',
                               -size=>'12',
-                              -onChange=>'focuslogin()',
+                              -onChange=>'focusloginbutton()',
                               -override=>'1');
    $html =~ s/\@\@\@PASSWORDFIELD\@\@\@/$temphtml/;
 
+   $temphtml = checkbox(-name=>'rememberme',
+                        -value=>'1',
+                        -checked=>cookie("openwebmail-rememberme")||0,
+                        -label=>'');
+   $html =~ s/\@\@\@REMEMBERMECHECKBOX\@\@\@/$temphtml/;
+
    if ( $#{$config{'domainnames'}} >0 && $config{'enable_domainselectmenu'} ) {
       $temphtml = popup_menu(-name=>'logindomain',
                              -values=>[@{$config{'domainnames'}}] );
@@ -175,7 +205,7 @@ sub loginmenu {
       $html =~ s/\@\@\@DOMAINEND\@\@\@/-->/;
    }
 
-   $temphtml = submit(-name =>"login",
+   $temphtml = submit(-name =>"loginbutton",
 		      -value=>"$lang_text{'login'}" );
 
    $html =~ s/\@\@\@LOGINBUTTON\@\@\@/$temphtml/;
@@ -206,7 +236,9 @@ sub login {
 
    if ($user eq "") {
       sleep $config{'loginerrordelay'};	# delayed response
-      openwebmailerror("$lang_err{'user_not_exist'}");
+      writelog("login error - no such user - loginname=$loginname");
+      # show 'pwd incorrect' instead of 'user not exist' for better security
+      openwebmailerror("$lang_err{'pwd_incorrect'}");	
    }
    if (! $config{'enable_rootlogin'}) {
       if ($user eq 'root' || $uuid==0) {
@@ -223,7 +255,7 @@ sub login {
       $errorcode=check_userpassword($user, $password);
    }
    if ( $errorcode==0 ) {
-      my $userconf="$config{'ow_etcdir'}/users.conf/$user";
+      my $userconf="$config{'ow_usersconfdir'}/$user";
       $userconf .= "\@$domain" if ($config{'auth_withdomain'});
       readconf(\%config, \%config_raw, "$userconf") if ( -f "$userconf");
 
@@ -283,7 +315,7 @@ sub login {
 
       if ( $config{'use_homedirspools'} || $config{'use_homedirfolders'} ) {
          my $mailgid=getgrnam('mail');
-         set_euid_egid_umask($uuid, $mailgid, 0077);	
+         set_euid_egid_umask($uuid, $mailgid, 0077);
          if ( $) != $mailgid) {	# egid must be mail since this is a mail program...
             openwebmailerror("Set effective gid to mail($mailgid) failed!");
          }
@@ -292,7 +324,7 @@ sub login {
       if ( $config{'use_homedirfolders'} ) {
          $folderdir = "$homedir/$config{'homedirfolderdirname'}";
       } else {
-         $folderdir = "$config{'ow_etcdir'}/users/$user";
+         $folderdir = "$config{'ow_usersdir'}/$user";
          $folderdir .= "\@$domain" if ($config{'auth_withdomain'});
       }
       ($folderdir =~ /^(.+)$/) && ($folderdir = $1);	# untaint
@@ -316,13 +348,13 @@ sub login {
 
       # create session file
       my $sessioncookie_value;
-      if ( -f "$config{'ow_etcdir'}/sessions/$thissession" ) { # continue an old session?
+      if ( -f "$config{'ow_sessionsdir'}/$thissession" ) { # continue an old session?
          $sessioncookie_value = cookie("$user-sessionid");
       } else {						       # a brand new sesion?
          $sessioncookie_value = crypt(rand(),'OW');
       }
-      open (SESSION, "> $config{'ow_etcdir'}/sessions/$thissession") or # create sessionid
-         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions/$thissession!");
+      open (SESSION, "> $config{'ow_sessionsdir'}/$thissession") or # create sessionid
+         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}/$thissession!");
       print SESSION $sessioncookie_value, "\n";
       print SESSION get_clientip(), "\n";
       close (SESSION);
@@ -417,6 +449,10 @@ sub login {
          $url="$config{'ow_cgiurl'}/openwebmail-main.pl?sessionid=$thissession&action=displayheaders_afterlogin";
       }
 
+      if ( ($ENV{'HTTPS'}=~/on/i || $ENV{'SERVER_PORT'}==443) && !$config{'stay_ssl_afterlogin'}) {
+         $url="http://$ENV{'HTTP_HOST'}$url" if ($url !~ m!https?://! );
+      }
+
       my @headers=();
       push(@headers, -pragma=>'no-cache');
 
@@ -424,13 +460,24 @@ sub login {
       my $cookie1 = cookie( -name  => "$user-sessionid",
                             -value => "$sessioncookie_value",
                             -path  => '/');
-      # cookie for other ap to find openwebmail loginname, expired until 3 month later
+      # cookie for other ap to find openwebmail loginname, expired until 1 month later
       my $cookie2 = cookie( -name  => "openwebmail-loginname",
                             -value => "$loginname",
                             -path  => '/',
                             -expires => "+1M" );
-      push(@headers, -cookie=>[$cookie1, $cookie2]);
+      # cookie for remember loginname switch, expired until 1 month later
+      my $cookie3 = cookie( -name  => "openwebmail-rememberme",
+                            -value => param("rememberme")||'',
+                            -path  => '/',
+                            -expires => "+1M" );
+      # cookie for ssl session, expired if not same session
+      my $cookie4 = cookie( -name  => "openwebmail-ssl",
+                            -value => ($ENV{'HTTPS'}=~/on/i ||
+                                       $ENV{'SERVER_PORT'}==443 ||
+                                       0),
+                            -path  => '/');
 
+      push(@headers, -cookie=>[$cookie1, $cookie2, $cookie3, $cookie4]);
       push(@headers, -charset=>$prefs{'charset'}) if ($CGI::VERSION>=2.57);
 
       # load page with Refresh header only if not MSIE on Mac
@@ -457,7 +504,7 @@ sub login {
 		qq|<font color="#cccccc" face="arial,helvetica,sans-serif" size=-1>\n|,
                 qq|$config{'name'} $config{'version'} $config{'releasedate'}<br><br>\n|,
 		qq|Copyright (C) 2001-2002<br>\n|,
-		qq|Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung<br><br>\n|,
+		qq|Chung-Kie Tung, Nai-Jung Kuo, Chao-Chiu Wang, Emir Litric, Thomas Chung, Dattola Filippo<br><br>\n|,
 		qq|Copyright (C) 2000<br>\n|,
 		qq|Ernie Miller  (original GPL project: Neomail)<br><br>\n|,
 		qq|</font></a>\n\n|,
@@ -501,13 +548,13 @@ sub login {
 
       if ( $config{'use_homedirspools'} || $config{'use_homedirfolders'} ) {
          my $mailgid=getgrnam('mail');
-         set_euid_egid_umask($uuid, $mailgid, 0077);	
+         set_euid_egid_umask($uuid, $mailgid, 0077);
       }
 
       if ( $config{'use_homedirfolders'} ) {
          $folderdir = "$homedir/$config{'homedirfolderdirname'}";
       } else {
-         $folderdir = "$config{'ow_etcdir'}/users/$user";
+         $folderdir = "$config{'ow_usersdir'}/$user";
          $folderdir .= "\@$domain" if ($config{'auth_withdomain'});
       }
 
@@ -522,18 +569,11 @@ sub login {
 
       sleep $config{'loginerrordelay'};	# delayed response
 
-      my $html = '';
-      printheader();
-      open (LOGINFAILED, "$config{'ow_etcdir'}/templates/$prefs{'language'}/loginfailed.template") or
-         openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/templates/$prefs{'language'}/loginfailed.template!");
-      while (<LOGINFAILED>) {
-         $html .= $_;
-      }
-      close (LOGINFAILED);
+      my $html=readtemplate("loginfailed.template");
+      $html = applystyle($html);
       $html =~ s/\@\@\@ERRORMSG\@\@\@/$errormsg/;
 
-      $html = applystyle($html);
-
+      printheader();
       print $html;
       printfooter(1);
       exit 0;
@@ -578,19 +618,20 @@ sub is_serverdomain_allowed {
 sub search_and_cleanoldsessions {
    my $oldcookie=$_[0];
    my $oldsessionid="";
-
    my $sessionid;
-   opendir (SESSIONSDIR, "$config{'ow_etcdir'}/sessions") or
-      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_etcdir'}/sessions!");
+   my @delfiles;
+
+   opendir (SESSIONSDIR, "$config{'ow_sessionsdir'}") or
+      openwebmailerror("$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}!");
    while (defined($sessionid = readdir(SESSIONSDIR))) {
       if ($sessionid =~ /^(.+\-session\-0.*)$/) {
          $sessionid = $1;
          if ($sessionid =~ /^$loginname\-session\-0./) { # remove user old session if timeout
-            if ( -M "$config{'ow_etcdir'}/sessions/$sessionid" > $prefs{'sessiontimeout'}/60/24 ) {
+            if ( -M "$config{'ow_sessionsdir'}/$sessionid" > $prefs{'sessiontimeout'}/60/24 ) {
                writelog("session cleanup - $sessionid");
-               unlink "$config{'ow_etcdir'}/sessions/$sessionid";
+               push(@delfiles, "$config{'ow_sessionsdir'}/$sessionid");
             } else {	# remove user old session from same client
-               open (SESSION, "$config{'ow_etcdir'}/sessions/$sessionid");
+               open (SESSION, "$config{'ow_sessionsdir'}/$sessionid");
                my $cookie = <SESSION>; chomp $cookie;
                my $ip = <SESSION>; chomp $ip;
                close (SESSION);
@@ -599,19 +640,20 @@ sub search_and_cleanoldsessions {
                      $oldsessionid=$sessionid;
                   } else {
                      writelog("session cleanup - $sessionid");
-                     unlink "$config{'ow_etcdir'}/sessions/$sessionid";
+                     push(@delfiles, "$config{'ow_sessionsdir'}/$sessionid");
                   }
                }
             }
          } else {	# remove others old session if more than 1 day
-            if ( -M "$config{'ow_etcdir'}/sessions/$sessionid" > 1 ) {
+            if ( -M "$config{'ow_sessionsdir'}/$sessionid" > 1 ) {
                writelog("session cleanup - $sessionid");
-               unlink "$config{'ow_etcdir'}/sessions/$sessionid";
+               push(@delfiles, "$config{'ow_sessionsdir'}/$sessionid");
             }
          }
       }
    }
    closedir (SESSIONSDIR);
+   unlink(@delfiles) if ($#delfiles>=0);
    return($oldsessionid);
 }
 ############## END SEARCH_AND_CLEANOLDSESSIONS ################
@@ -622,8 +664,8 @@ sub releaseupgrade {
    my ($folderdir, $user_releasedate)=@_;
    my $content;
    my $rc_upgrade=0;
-   my ($_OFFSET, $_FROM, $_TO, $_DATE, $_SUBJECT, $_CONTENT_TYPE, $_STATUS, $_SIZE, $_REFERENCES)
-       =(0,1,2,3,4,5,6,7,8);
+   my ($_OFFSET, $_FROM, $_TO, $_DATE, $_SUBJECT, $_CONTENT_TYPE, $_STATUS, $_SIZE, $_REFERENCES, $_CHARSET)
+       =(0,1,2,3,4,5,6,7,8,9);
 
    if ( $user_releasedate lt "20011101" ) {
       if ( -f "$folderdir/.filter.book" ) {
@@ -734,13 +776,12 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
-
-         next if ( ! -f "$headerdb$config{'dbm_ext'}");
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
 
          filelock($folderfile, LOCK_SH);
          open (FOLDER, $folderfile);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
+         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
 
          if ( $HDB{'METAINFO'} eq metainfo($folderfile) ) { # upgrade only if hdb is uptodate
             @messageids=keys %HDB;
@@ -785,7 +826,7 @@ sub releaseupgrade {
             }
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
          filelock($folderfile, LOCK_UN);
       }
       writehistory("release upgrade - $folderdir/* by 20020108.02");
@@ -799,11 +840,10 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
 
-         next if ( ! -f "$headerdb$config{'dbm_ext'}");
-
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
+         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
 
          @messageids=keys %HDB;
          foreach my $id (@messageids) {
@@ -826,7 +866,7 @@ sub releaseupgrade {
             $HDB{$id}=join('@@@', @attr);
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
 
          my $cachefile="$headerdb.cache";
          ($cachefile =~ /^(.+)$/) && ($cachefile = $1);  # untaint ...
@@ -845,12 +885,12 @@ sub releaseupgrade {
       foreach my $foldername (@validfolders) {
          my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
          my (%HDB, @messageids, @attr);
-         next if ( ! -f "$headerdb$config{'dbm_ext'}");
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
 
          filelock($folderfile, LOCK_SH);
          open (FOLDER, $folderfile);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX);
-         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", undef);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
+         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
 
          @messageids=keys %HDB;
          foreach my $id (@messageids) {
@@ -886,14 +926,47 @@ sub releaseupgrade {
             $HDB{$id}=join('@@@', @attr);
          }
          dbmclose(%HDB);
-         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
          filelock($folderfile, LOCK_UN);
       }
       writehistory("release upgrade - $folderdir/* by 20020601");
       writelog("release upgrade - $folderdir/* by 20020601");
    }
 
-   if ( $user_releasedate lt "20020708" ) {
+   if ( $user_releasedate lt "20021111" ) {
+      my (@validfolders, $folderusage);
+      getfolders(\@validfolders, \$folderusage);
+
+      foreach my $foldername (@validfolders) {
+         my ($folderfile, $headerdb)=get_folderfile_headerdb($user, $foldername);
+         my (%HDB, @messageids, @attr);
+         next if ( ! -f "$headerdb$config{'dbm_ext'}" || -z "$headerdb$config{'dbm_ext'}" );
+
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_EX) if (!$config{'dbmopen_haslock'});
+         dbmopen (%HDB, "$headerdb$config{'dbmopen_ext'}", 0600);
+
+         @messageids=keys %HDB;
+         foreach my $id (@messageids) {
+            next if ( $id eq 'METAINFO'
+             || $id eq 'NEWMESSAGES'
+             || $id eq 'INTERNALMESSAGES'
+             || $id eq 'ALLMESSAGES'
+             || $id eq "" );
+            @attr=split( /@@@/, $HDB{$id} );
+            if ( $attr[$_CHARSET] eq "" &&
+                 $attr[$_CONTENT_TYPE]=~/charset="?([^\s"';]*)"?\s?/i) {
+               $attr[$_CHARSET]=$1;
+               $HDB{$id}=join('@@@', @attr);
+            }
+         }
+         dbmclose(%HDB);
+         filelock("$headerdb$config{'dbm_ext'}", LOCK_UN) if (!$config{'dbmopen_haslock'});
+      }
+      writehistory("release upgrade - $folderdir/.*$config{'dbm_ext'} by 20021111.02");
+      writelog("release upgrade - $folderdir/.*$config{'dbm_ext'} by 20021111.02");
+   }
+
+   if ( $user_releasedate lt "20021116" ) {
       $rc_upgrade=1;	# .openwebmailrc upgrade will be requested
    }
 
