@@ -50,7 +50,7 @@ sub get_userinfo {
 }
 
 
-sub get_userlist {      # only used by checkmail.pl -a
+sub get_userlist {      # only used by openwebmail-tool.pl -a
    my @userlist=();
 
    my $list = $ldap->search (
