@@ -1,5 +1,9 @@
 #
 # auth_pop3.pl - authenticate user with POP3 server
+#
+# 2002/03/08 tung@turtle.ee.ncku.edu.tw
+#
+
 #
 # This module assumes that
 #
@@ -64,8 +68,6 @@
 # $pop3_authport: the port which pop3 server is listening to
 # $local_uid: uid used on this machine
 #
-# 2002/03/08 tung@turtle.ee.ncku.edu.tw
-#
 
 # global vars, also used by openwebmail.pl
 use vars qw($pop3_authserver $pop3_authport $local_uid);
@@ -98,7 +100,7 @@ sub get_userinfo {
 }
 
 
-sub get_userlist {	# only used by checkmail.pl -a
+sub get_userlist {	# only used by openwebmail-tool.pl -a
    return();		# not supported, return empty
 }
 
@@ -137,8 +139,8 @@ sub check_userpassword {
       (close($remote_sock) && return(-2)) if (/^\-/);		# username error
       print $remote_sock &encode_base64($password);
       $_=<$remote_sock>;
-      (close($remote_sock) && return(-4)) if (/^\-/);		# passwd error
-   } else {
+   }
+   if (! /^\+/) {	# not supporting auth login or auth login failed
       print $remote_sock "user $user\r\n";
       $_=<$remote_sock>;
       (close($remote_sock) && return(-2)) if (/^\-/);		# username error
