@@ -1,8 +1,9 @@
 #
-# filelock.pl - functions for filelock on local or nfs server
+# filelock.pl - filelock routines for local or nfs server
 #
 # 2001/04/25 tung@turtle.ee.ncku.edu.tw
 #
+
 use strict;
 use Fcntl qw(:DEFAULT :flock);
 use FileHandle;
@@ -25,8 +26,9 @@ sub filelock_flock {
    my ($filename, $lockflag)=@_;
    my ($dev, $inode, $fh);
 
+   ($filename =~ /^(.+)$/) && ($filename = $1);	# untaint...
+
    if ( (! -e $filename) && $lockflag ne LOCK_UN) {
-      ($filename =~ /^(.+)$/) && ($filename = $1);
       sysopen(F, $filename, O_RDWR|O_CREAT, 0600); # create file for lock
       close(F);
    }
@@ -36,7 +38,7 @@ sub filelock_flock {
       return(0);
    }
 
-   if (defined($opentable{"$dev-$inode"}) ) {	
+   if (defined($opentable{"$dev-$inode"}) ) {
       $fh=$opentable{"$dev-$inode"};
    } else { # handle not found, open it!
       $fh=FileHandle->new();
@@ -57,7 +59,7 @@ sub filelock_flock {
       eval {
          local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
          alarm 60;
-         $retval=flock($fh, $lockflag & (~LOCK_NB) );	
+         $retval=flock($fh, $lockflag & (~LOCK_NB) );
          alarm 0;
       };
       if ($@) {	# eval error, it means timeout
@@ -65,7 +67,7 @@ sub filelock_flock {
       }
       return($retval);
 
-   } else {			# blocking lock				
+   } else {			# blocking lock
       return(flock($fh, $lockflag));
    }
 }
@@ -75,6 +77,12 @@ sub filelock_flock {
 # it is only recommended if the files are located on remote nfs server
 # and the lockd on your nfs server or client has problems
 # since it is slower than flock
+#
+# This routine stores the shared lock counter in filename.lock and
+# uses filename.lock.lock to guarentee unique access to filename.lock.
+# If the filename is locates in a readonly directory, since the
+# filename.lock and filename.lock.lock could not be created, this routine will
+# grant all shared lock and deny all exclusive lock.
 sub filelock_dotlockfile {
    my ($filename, $lockflag)=@_;
    my ($mode, $count);
@@ -91,7 +99,9 @@ sub filelock_dotlockfile {
    my $oldumask=umask(0111);
 
    # resolve symbolic link
+   my $ldepth=0;
    while (-l "$filename") {
+      $ldepth++; return (0) if ($ldepth>32);		# link to deep
       $filename=readlink($filename);
    }
    ($filename =~ /^(.+)$/) && ($filename = $1);		# untaint ...
@@ -104,9 +114,16 @@ sub filelock_dotlockfile {
          unlink("$filename.lock") if (time()-$t > 300);
       }
 
-      if (_lock("$filename.lock")==0) {
+      my $locklock=_lock("$filename.lock");
+      if ($locklock==0) {
          sleep 1;
          next;
+      } elsif ($locklock==-1) {	# rdonly dir, no further processing
+         if ($lockflag & LOCK_EX) {
+            return(0);
+         } else {
+            return(1);
+         }
       }
 
       if ( $lockflag & LOCK_UN ) {
@@ -197,7 +214,11 @@ sub _lock {
       close(LL);
       return(1)
    } else {
-      return(0);
+      if ($!=~/permission/i) {	# .lock file in readonly dir?
+         return(-1);
+      } else {			# .lock file already exist
+         return(0);
+      }
    }
 }
 sub _unlock {
@@ -207,7 +228,11 @@ sub _unlock {
    if ( unlink("$filename.lock") ) {
       return(1);
    } else {
-      return(0);
+      if ($!=~/permission/i) {	# .lock file in readonly dir?
+         return(-1);
+      } else {
+         return(0);
+      }
    }
 }
 
